﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Windows.Media.Animation;
using System.Windows.Markup;
using System.Windows.Controls.Primitives;

namespace UmlControlLibrary
{
    public enum UmlRelationType { Inheritance, Association, Composition, Aggregate };
    
    public class UmlArrow : SelectableItemControl
    {
        public string Label
        {
            get;
            set;
        }
        public PathFigure Figure
        {
            get;
            set;
        }

        public Point P1
        {
            get;
            set;
        }

        public Point P2
        {
            get;
            set;
        }

        public virtual PathFigure CalculateArrow(out Point correctedPoint)
        {
            PathFigure pathfig = this.Figure;
            correctedPoint = P2;
            return pathfig;
        }

        public virtual void UpdateArrow(Point p1, Point p2)
        {
            P1 = p1;
            P2 = p2;
            if ((p1.X != p2.X) || (p1.Y != p2.Y))
                CalculateArrow(out p2);
            else
                Figure.Segments.Clear();
            P2 = p2;
        }

        public UmlArrow(string label, Point p1, Point p2)
            : base()
        {
            Figure = new PathFigure();
            this.P1 = p1;
            this.P2 = p2;
            CalculateArrow(out p2);
            this.P2 = p2;
        }
    }

    public class UmlArrowInheritance : UmlArrow
    {
        public UmlArrowInheritance(string label, Point p1, Point p2)
            : base(label, p1, p2)
        {

        }

        public override PathFigure CalculateArrow(out Point correctedPoint)
        {
            PathFigure pathfig = this.Figure;

            double ArrowLength = 15;
            double ArrowAngle = 45;
            bool IsArrowClosed = true;

            Matrix matx = new Matrix();
            Vector vect = P1 - P2;
            vect.Normalize();
            vect *= ArrowLength;

            PolyLineSegment polyseg;
            if (pathfig.Segments.Count == 0)
            {
                polyseg = new PolyLineSegment();
                pathfig.Segments.Add(polyseg);
            }
            else
                polyseg = (PolyLineSegment)pathfig.Segments[0];
            polyseg.Points.Clear();
            matx.Rotate(ArrowAngle / 2);
            //            pathfig.StartPoint = (pt2 - vect) + vect * matx;
            //            polyseg.Points.Add((pt2 - vect));
            pathfig.StartPoint = P2 + vect * matx;
            polyseg.Points.Add(P2);

            matx.Rotate(-ArrowAngle);
            //            polyseg.Points.Add((pt2 - vect) + vect * matx);
            polyseg.Points.Add(P2 + vect * matx);
            pathfig.IsClosed = IsArrowClosed;
            pathfig.IsFilled = true;

            correctedPoint = P2 + vect;

            return pathfig;
        }
    }

    public class UmlArrowDiamond : UmlArrow
    {
        public UmlArrowDiamond(string label, Point p1, Point p2):base(label, p1, p2)
        {

        }
        public override PathFigure CalculateArrow(out Point correctedPoint)
        {
            PathFigure pathfig = this.Figure;

            double ArrowLength = 10;
            double ArrowAngle = 45;
            bool IsArrowClosed = true;

            Matrix matx = new Matrix();
            Vector vect = P1 - P2;
            vect.Normalize();
            vect *= ArrowLength;

            PolyLineSegment polyseg ;
            if (pathfig.Segments.Count == 0)
            {
                polyseg = new PolyLineSegment();
                pathfig.Segments.Add(polyseg);
            }
            else
                polyseg = (PolyLineSegment)pathfig.Segments[0];
            polyseg.Points.Clear();
            matx.Rotate(ArrowAngle / 2);
            //            pathfig.StartPoint = (pt2 - vect) + vect * matx;
            //            polyseg.Points.Add((pt2 - vect));
            pathfig.StartPoint = P2 + vect * matx;
            polyseg.Points.Add(P2);

            matx.Rotate(-ArrowAngle);
            //            polyseg.Points.Add((pt2 - vect) + vect * matx);

            polyseg.Points.Add(P2 + vect * matx);

            correctedPoint = pathfig.StartPoint + vect * matx;
            polyseg.Points.Add(correctedPoint);

            pathfig.IsClosed = IsArrowClosed;
            pathfig.IsFilled = true;           

            return pathfig;
        }
    }


    public class LineHelper
    {
        public enum Side
        {
            Top,
            Bottom,
            Left,
            Right
        }
        public enum LineDirection { Horizontal, Vertical };
        public static Point Interpolate(Point p1, Point p2, double factor)
        {
            return new Point(p1.X + (p2.X - p1.X) * factor, p1.Y + (p2.Y - p1.Y) * factor);
        }

        public static PolyLineSegment GetOrthogonalSegment(Point p1, Point pc1, Point pc2, Point p2, Side side1, Side side2)
        {
            LineDirection dirP1 = GetDirection(side1);
            LineDirection dirP2 = GetDirection(side2);
            Point pc = LineHelper.Interpolate(pc1, pc2, 0.5);
            PointCollection pointCol = new PointCollection();
            if (dirP1 == LineHelper.LineDirection.Horizontal)
                pointCol.Add(new Point(pc.X, p1.Y));
            else
                pointCol.Add(new Point(p1.X, pc.Y));

            pointCol.Add(pc);

            if (dirP2 == LineHelper.LineDirection.Horizontal)
                pointCol.Add(new Point(pc.X, p2.Y));
            else
                pointCol.Add(new Point(p2.X, pc.Y));

            pointCol.Add(p2);

            return new PolyLineSegment(pointCol, true);
        }
        public static LineDirection GetDirection(Side side)
        {
            return ((side == Side.Left) || (side == Side.Right)) ? LineDirection.Horizontal : LineDirection.Vertical;
        }
    }

    public class UmlEnd
    {
        public UmlEnd(PathGeometry ownerGeometry)
        {
            this.Geometry = ownerGeometry;
            LabelControl = new DraggableLabel();            
            Binding binding = new Binding("Label");
            binding.Mode = BindingMode.TwoWay;
            binding.Source = this;
            BindingOperations.SetBinding(LabelControl,DraggableLabel.TextProperty, binding);
        }

        public UmlControl EndControl
        {
            get;
            set;
        }

        PathGeometry Geometry
        {
            get;
            set;
        }

        UmlArrow _arrow = null;
        public UmlArrow Arrow
        {
            get {return _arrow;}
            set
            {
                if (_arrow != null)
                    Geometry.Figures.Remove(_arrow.Figure);
                _arrow = value;
                    Geometry.Figures.Add(_arrow.Figure);
            }
        }

        string _label;
        public string Label
        {
            get { return _label;}
            set
            {
                _label = value;
                LabelControl.GetBindingExpression(DraggableLabel.TextProperty).UpdateTarget();
            }
        }

        public DraggableLabel LabelControl
        {
            get;
            set;
        }

        public LineHelper.Side Side
        {
            get;
            set;
        }

        public void RefreshLabel()
        {
            double x = Arrow.P2.X;
            double y = Arrow.P2.Y;

            switch (Side)
            {
                case LineHelper.Side.Bottom:
                    x+=5;
                    y+=5;
                    break;
                case LineHelper.Side.Top:
                    x+=5;
                    y-=(5 + LabelControl.ActualHeight);
                    break;
                case LineHelper.Side.Left:
                    x-=(5 + LabelControl.ActualWidth);
                    y+=5;
                    break;
                case LineHelper.Side.Right:
                    x+=5;
                    y+=5;
                    break;
            }

            Canvas.SetLeft(LabelControl, x);
            Canvas.SetTop(LabelControl, y);
        }
    }

    public class UmlRelation : SelectableItemControl
    {
        const int HalfThumbSize = 3;
        const int ThumbSize = 6;

        public UmlEnd End1
        {
            get;
            set;
        }
        public UmlEnd End2
        {
            get;
            set;
        }

        string _label;
        public string Label
        {
            get { return _label; }
            set
            {
                _label = value;
                LabelControl.GetBindingExpression(DraggableLabel.TextProperty).UpdateTarget();
            }
        }

        public DraggableLabel LabelControl
        {
            get;
            set;
        }

        public List<Point> RelationLinePoints
        {
            get;
            set;
        }

        PathFigure _figure;
        public PathFigure LineFigure
        {
            get
            {
                return _figure;
            }
            set{
                if (Geometry.Figures.Contains(_figure))
                    Geometry.Figures[Geometry.Figures.IndexOf(_figure)] = value;
                else
                    Geometry.Figures.Add(value);
                _figure = value;
                _figure.IsFilled = false;
            }
        }
        public PathGeometry Geometry
        {
            get;
            set;
        }

        public Path RelationGraphPath
        {
            get;
            set;
        }

        public Point GetCenter(Control e)
        {
            double l = Canvas.GetLeft(e),
                   r = Canvas.GetRight(e),
                   t = Canvas.GetTop(e),
                   b = Canvas.GetBottom(e);
            if (double.IsNaN(r))
            {
                r = l + e.ActualWidth;
                b = t + e.ActualHeight;
            }
            return new Point((l+r)/2,(t+b)/2);
        }

        private double getMagnitude(Point l1, Point l2)
        {
            double x, y;
            x = l2.X - l1.X; x *= x;
            y = l2.Y - l1.Y; y *= y;
            return x + y;
        }

        private double DistanceFromLine(Point p, Point l1, Point l2)
        {
            const double EPS = 0.000001;
            const double EPSEPS = EPS * EPS;
            double SqLineMag, u, ix, iy;
            double result;

            SqLineMag = getMagnitude(l1, l2);
            if (SqLineMag < EPSEPS)
                return -1;

            u = ((p.X - l1.X) * (l2.X - l1.X) + (p.Y - l1.Y) * (l2.Y - l1.Y)) / SqLineMag;
            if ((u < EPS) || (u > 1))
            {
                ix = getMagnitude(p, l1);
                iy = getMagnitude(p, l2);
                result = Math.Min(ix, iy);
            }
            else
            {
                ix = l1.X + u * (l2.X - l1.X);
                iy = l1.Y + u * (l2.Y - l1.Y);
                result = getMagnitude(p, new Point(ix, iy));
            }
            return Math.Sqrt(result);
        }

        public bool IsPointOnSegment(Point p, Point p1, Point p2, out double factor, double tolerance, double maxDist)
        {
            if (p == p1)
            {
                factor = 0;
                return true;
            }
            if (p == p2)
            {
                factor = 1;
                return true;
            }
            
            Vector v1 = p2 - p1;
            Vector v2 = p - p1;
            Vector v1n = v1; v1n.Normalize();
            Vector v2n = v2; v2n.Normalize();

            double dot = v1n.X * v2n.X + v1n.Y * v2n.Y;
            if ((dot >= 1-tolerance) && (dot <= 1 + tolerance))
            {
                factor = v2.Length / v1.Length;
                if ((factor>=0) && (factor<=1) && (DistanceFromLine(p,p1,p2)<maxDist))
                    return true;
            }
            factor = 0;
            return false;
        }

        public int GetSegmentPart(Point p, out double factor)
        {
            PolyLineSegment pls = (PolyLineSegment)LineFigure.Segments[0];
            factor = 0;
            for (int i = 0; i < pls.Points.Count - 1; i++)
            {
                Point p1 = pls.Points[i];
                Point p2 = pls.Points[i + 1];

                if (IsPointOnSegment(p, p1,p2,out factor, 0.001, 2))
                    return i;
            }
            return -1;
        }

        protected void SetLabelPos(Point p)
        {
            if (LabelControl != null)
            {
                Canvas.SetLeft(LabelControl, p.X - LabelControl.ActualWidth / 2);
                Canvas.SetTop(LabelControl, p.Y + 5);
            }
        }

        public void RecalcRelationLink()
        {
            RecalcRelationLink(null);
        }

        public void RecalcRelationLink(UmlControl refreshEnd)
        {
            Point c1, c2;
            PolyLineSegment pls;

            // initial relation calculation
            if (LineFigure.Segments.Count == 0)
            {
                c2 = End2.EndControl.MagnetPoint(GetCenter(End1.EndControl));
                c1 = End1.EndControl.MagnetPoint(c2);
            }
            else{
                PointCollection pointCol = (LineFigure.Segments[0] as PolyLineSegment).Points;
                c1 = pointCol[1];
                c2 = pointCol[pointCol.Count - 2];

                if (pointCol.Count == 2)
                {
                    c1 = End2.EndControl.MagnetPoint(c1);
                    c2 = End1.EndControl.MagnetPoint(c2);
                }
                /*  if (pointCol.Count>1)
                    c2 = pointCol[pointCol.Count-2];
                else
                    c2 = LineFigure.StartPoint;*/
            }


            LineHelper.Side side;
            Point p1 = End1.EndControl.MagnetPoint(c1, out side);
            End1.Side = side;
            Point p2 = End2.EndControl.MagnetPoint(c2, out side);
            End2.Side = side;

            LineFigure.StartPoint = p1;

            if (LineFigure.Segments.Count == 0)
            {
                PointCollection pointCol = new PointCollection();
                pointCol.Add(p1);
                pointCol.Add(p2);
                pls = new PolyLineSegment(pointCol, true);
                LineFigure.Segments.Add(pls);
                RecalcRelationLink();
            }
            else
            {
                pls = LineFigure.Segments[0] as PolyLineSegment;
                PointCollection pointCol = pls.Points;
                pointCol[0] = p1;
                pointCol[pointCol.Count - 1] = p2;

                /*            Point pc1 = LineHelper.Interpolate(p1, p2, 1.0f / 3);
                            Point pc2 = LineHelper.Interpolate(p1, p2, 2.0f / 3);

                            PolyLineSegment pls = LineHelper.GetOrthogonalSegment(p1, pc1, pc2, p2, End1.Side, End2.Side);

                            if (LineFigure.Segments.Count > 0)
                                LineFigure.Segments[0] = pls;
                            else
                                LineFigure.Segments.Add(pls);

                            */


                if (End1.Arrow != null)
                {
                    End1.Arrow.UpdateArrow(pls.Points[1], p1);
                    LineFigure.StartPoint = End1.Arrow.P2;
                    End1.RefreshLabel();
                }
                if (End2.Arrow != null)
                {
                    End2.Arrow.UpdateArrow(pls.Points[pls.Points.Count - 2], p2);
                    pls.Points[pls.Points.Count - 1] = End2.Arrow.P2;
                    End2.RefreshLabel();
                }

                Point center = LineHelper.Interpolate(p1, p2, 0.5);

                SetLabelPos(center);

                if (thumbs.Count > 0)
                {
                    Canvas.SetLeft(thumbs[0], p1.X - HalfThumbSize);
                    Canvas.SetTop(thumbs[0], p1.Y - HalfThumbSize);
                    Canvas.SetLeft(thumbs[thumbs.Count - 1], p2.X - HalfThumbSize);
                    Canvas.SetTop(thumbs[thumbs.Count - 1], p2.Y - HalfThumbSize);
                }
            }
        }

        public void SetRelationType(UmlRelationType relationType)
        {
            PolyLineSegment seg = (LineFigure.Segments[0] as PolyLineSegment);
            Point s1 = seg.Points[0],
                  s2 = LineFigure.StartPoint, 
                  e1 = seg.Points[seg.Points.Count - 2],
                  e2 = seg.Points[seg.Points.Count - 1];

            switch (relationType)
            {
                case UmlRelationType.Association:
                    End1.Arrow = new UmlArrow(End1.Label, s1, s2);
                    End2.Arrow = new UmlArrow(End2.Label, e1, e2);
                    RelationGraphPath.Fill = Brushes.Black;
                    break;
                case UmlRelationType.Inheritance:
                    End1.Arrow = new UmlArrow(End1.Label, s1, s2);
                    End2.Arrow = new UmlArrowInheritance(End2.Label, e1, e2);
                    RelationGraphPath.Fill = Brushes.White;
                    break;
                case UmlRelationType.Aggregate:
                    End1.Arrow = new UmlArrow(End1.Label, s1, s2);
                    End2.Arrow = new UmlArrowDiamond(End2.Label, e1, e2);
                    RelationGraphPath.Fill = Brushes.White;
                    break;
                case UmlRelationType.Composition:
                    End1.Arrow = new UmlArrow(End1.Label, s1, s2);
                    End2.Arrow = new UmlArrowDiamond(End2.Label, e1, e2);
                    RelationGraphPath.Fill = Brushes.Black;
                    break;
            }

            RecalcRelationLink();
        }

        private void AddControlPoint(int segment, double factor)
        {
            PointCollection pointCol = (LineFigure.Segments[0] as PolyLineSegment).Points;
            Point p = LineHelper.Interpolate(pointCol[segment], pointCol[segment + 1], factor);
            pointCol.Insert(segment + 1, p);
            if (Selected)
            {
                Thumb t = AddThumb(p);
                thumbs.Insert(segment+1,t);
                OwnerElement.AddChild(t);
            }
        }

        void t_DragDelta(object sender, DragDeltaEventArgs e)
        {
            int count = thumbs.Count;

            Thumb item = sender as Thumb;
            int index = thumbs.IndexOf(item);
            if ((index > 0) && (index < count - 1))
            {
                Point p;
                PointCollection controlPoints = (LineFigure.Segments[0] as PolyLineSegment).Points;
                p = controlPoints[index];

                double X = SelectableItemControl.AlignToGrid(p.X + e.HorizontalChange);
                double Y = SelectableItemControl.AlignToGrid(p.Y + e.VerticalChange);

                p = new Point(X, Y);

                /*   if (index == count)
                   {
                       p = umlLine.end2.MagnetPoint(p);
                       X = p.X;
                       Y = p.Y;
                   }*/

                Canvas.SetLeft(item, X - HalfThumbSize);
                Canvas.SetTop(item, Y - HalfThumbSize);

                /*     if (index == 0)
                     {
                         _startPoint = p;
                         BindingOperations.GetBindingExpression(pf, PathFigure.StartPointProperty).UpdateTarget();
                         BindingOperations.GetBindingExpression(controlLines[0], Line.X1Property).UpdateTarget();
                         BindingOperations.GetBindingExpression(controlLines[0], Line.Y1Property).UpdateTarget();
                     }
                     else
                     {*/
                controlPoints[index] = p;
                //       BindingOperations.GetBindingExpression(bs, PolyBezierSegment.PointsProperty).UpdateTarget();
                /*                    BindingOperations.GetBindingExpression(controlLines[index - 1], Line.X2Property).UpdateTarget();
                                    BindingOperations.GetBindingExpression(controlLines[index - 1], Line.Y2Property).UpdateTarget();
                                    if (index < count)
                                    {
                                        BindingOperations.GetBindingExpression(controlLines[index], Line.X1Property).UpdateTarget();
                                        BindingOperations.GetBindingExpression(controlLines[index], Line.Y1Property).UpdateTarget();
                                    }

                                    if (index >= count - 1)
                                    {
                                        GraphViz.CalculateArrow(endArrow, _controlPoints[count - 2], _controlPoints[count - 1]);
                                    }
                                }
                  */
                RecalcRelationLink();
            }
            IsDragging = true;
            e.Handled = true;
        }

        private Thumb AddThumb(Point p)
        {
            Thumb t = new Thumb();
            t.Height = ThumbSize;
            t.Width = ThumbSize;
            t.BorderBrush = Brushes.Blue;
            t.BorderThickness = new Thickness(HalfThumbSize);
            Canvas.SetLeft(t, p.X - HalfThumbSize);
            Canvas.SetTop(t, p.Y - HalfThumbSize);
            t.DragDelta += new DragDeltaEventHandler(t_DragDelta);
            
            //t.PreviewMouseLeftButtonDown += new System.Windows.Input.MouseButtonEventHandler(t_PreviewMouseDown);
            //t.PreviewMouseUp += new System.Windows.Input.MouseButtonEventHandler(t_PreviewMouseUp);
            t.PreviewMouseLeftButtonUp += new MouseButtonEventHandler(t_PreviewMouseLeftButtonUp);

            return t;
        }

        void t_PreviewMouseLeftButtonUp(object sender, MouseButtonEventArgs e)
        {
            if (IsDragging)
            {
                Thumb t = sender as Thumb;
                for (int index=thumbs.Count - 2;index>0;index--)
                {
                    PolyLineSegment pls = (PolyLineSegment)LineFigure.Segments[0];
                    Point p1 = pls.Points[index - 1];
                    Point p2 = pls.Points[index + 1];
                    Point p = pls.Points[index];
                    double factor = 0;
                    if (IsPointOnSegment(p, p1, p2, out factor, 0.001, 10))
                    {
                        pls.Points.RemoveAt(index);
                        t = thumbs[index];
                        this.OwnerElement.RemoveChild(t);
                        thumbs.RemoveAt(index);
                    }
                }
            }
            IsDragging = false;
        }

        bool IsDragging = false;

        List<Thumb> thumbs = new List<Thumb>();

        private void CleanThumbs()
        {
            foreach (Thumb t in thumbs)
                this.OwnerElement.RemoveChild(t);
            thumbs.Clear();
        }

        private void Hilight(bool hilight)
        {
            if (hilight)
            {
                this.RelationGraphPath.StrokeThickness = 2;
                this.RelationGraphPath.Stroke = Brushes.Gray;
            }
            else
            {
                this.RelationGraphPath.StrokeThickness = 1;
                this.RelationGraphPath.Stroke = Brushes.Black;
            }
        }

        public override void Select()
        {
            base.Select();
            CleanThumbs();

            foreach (Point p in (LineFigure.Segments[0] as PolyLineSegment).Points)
                thumbs.Add(AddThumb(p));
            foreach(Thumb t in thumbs)
                this.OwnerElement.AddChild(t);
            Hilight(true);
        }

        public override void Deselect()
        {
            base.Deselect();
            CleanThumbs();
            Hilight(false);
        }

        public UmlRelation(UmlControl EndControl1, UmlControl EndControl2, UmlRelationType relationType):base()
        {
            this.Geometry = new PathGeometry();
            LineFigure = new PathFigure();           
            this.End1 = new UmlEnd(this.Geometry);
            this.End2 = new UmlEnd(this.Geometry);
            this.End1.EndControl = EndControl1;
            this.End2.EndControl = EndControl2;

            if (!this.End1.EndControl.Relations.Contains(this))
                this.End1.EndControl.Relations.Add(this);

            if (!this.End2.EndControl.Relations.Contains(this))
                this.End2.EndControl.Relations.Add(this);

            RecalcRelationLink();

            this.RelationGraphPath = new Path();
            this.RelationGraphPath.Stroke = Brushes.Black;
            this.RelationGraphPath.Fill = Brushes.Black;
            this.RelationGraphPath.DataContext = this;
            this.RelationGraphPath.SetBinding(Path.DataProperty, "Geometry");

            this.OwnerElement = this.End1.EndControl.OwnerElement;
            this.OwnerElement.AddChild(this.RelationGraphPath);

            this.OwnerElement.AddChild(this.End1.LabelControl);
            this.OwnerElement.AddChild(this.End2.LabelControl);

            LabelControl = new DraggableLabel();
            Binding binding = new Binding("Label");
            binding.Mode = BindingMode.TwoWay;
            binding.Source = this;
            BindingOperations.SetBinding(LabelControl, DraggableLabel.TextProperty, binding);

            this.OwnerElement.AddChild(this.LabelControl);

            /*this.End1.RefreshLabel();
            this.End2.RefreshLabel();*/

            SetRelationType(relationType);

            this.RelationGraphPath.MouseLeftButtonDown += new MouseButtonEventHandler(UmlRelation_MouseLeftButtonDown);
            this.End1.LabelControl.MouseLeftButtonDown += new MouseButtonEventHandler(UmlRelation_MouseLeftButtonDown);
            this.End2.LabelControl.MouseLeftButtonDown += new MouseButtonEventHandler(UmlRelation_MouseLeftButtonDown);
            this.LabelControl.MouseLeftButtonDown += new MouseButtonEventHandler(UmlRelation_MouseLeftButtonDown);
            this.RelationGraphPath.MouseEnter += new MouseEventHandler(RelationGraphPath_MouseEnter);
            this.RelationGraphPath.MouseLeave += new MouseEventHandler(RelationGraphPath_MouseLeave);
        }

        void RelationGraphPath_MouseLeave(object sender, MouseEventArgs e)
        {
            if (!Selected)
                Hilight(false);
        }

        void RelationGraphPath_MouseEnter(object sender, MouseEventArgs e)
        {
            Hilight(true);
        }

        void UmlRelation_MouseLeftButtonDown(object sender, MouseButtonEventArgs e)
        {
            if (!this.Selected)
                Select();
            else
            {
                // find if we clicked in a part of the relation line
                double factor;
                Point mousePos;
                mousePos = this.OwnerElement.GetMousePos(e);
                int SegmentPart = GetSegmentPart(mousePos, out factor);
                if (SegmentPart >= 0)
                    AddControlPoint(SegmentPart, factor);
            }
            e.Handled = true;
        }
    }
}
